xquery version "3.1";

module namespace website="http://ahiqar.uni-goettingen.de/ns/website";

import module namespace commons="http://ahiqar.uni-goettingen.de/ns/commons" at "commons.xqm";
import module namespace rest="http://exquery.org/ns/restxq";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace tgmd="http://textgrid.info/namespaces/metadata/core/2010";
declare namespace xhtml="http://www.w3.org/1999/xhtml";
declare namespace output="http://www.w3.org/2010/xslt-xquery-serialization";
declare namespace rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#";

declare
    %rest:GET
    %rest:HEAD
    %rest:path("/website/manuscripts")
    %output:method("xml")
function website:manuscripts() {
    let $syc := local:manuscripts-lang("syc")
    let $ara := local:manuscripts-lang("ara")

    return
        element xhtml:div {
            element xhtml:h2 { "The Syriac witnesses" },
            $syc,
            element xhtml:h2 { "The Arabic and Karshuni witnesses" },
            $ara
        }
};

declare function local:manuscripts-lang($lang as xs:string) {
    let $syriac := $lang eq "syc"
    let $lang := if($syriac) then "syriac" else "arabic-karshuni"
    let $teiHeaders :=
        let $syrAgg := "textgrid:3r84g"
        let $araAgg := "textgrid:3r9ps"
        let $karshuniAgg := "textgrid:3r84h"
        return
            if($syriac) then
                commons:get-available-aggregates("3r84g")
                ! commons:get-child-tei-uri(.)
                ! doc(.)//tei:teiHeader
            else
                (commons:get-available-aggregates("3r9ps"), commons:get-available-aggregates("3r84h"))
                ! commons:get-child-tei-uri(.)
                ! doc(.)//tei:teiHeader

    return
    element xhtml:div {
        attribute class { "manuscripts" },
        for $teiHeader at $pos in $teiHeaders
        where $pos
        let $meta := doc( $commons:meta || ($teiHeader/base-uri() => tokenize("/"))[last()])
        let $tgUri := ($teiHeader/base-uri() => tokenize("/"))[last()] => substring-before(".xml")
        let $msUri := $tgUri => commons:get-parent-uri()
        let $firstpage := string(($teiHeader/root()//tei:pb[@facs][@n])[1]/@n)
        let $url := "https://ahiqar.uni-goettingen.de/" || $lang || "/#/?source=external&amp;redirectUrl=https%3A%2F%2Fahiqar.uni-goettingen.de%2Fwebsite%2F&amp;itemurl=https%3A%2F%2Fahiqar.uni-goettingen.de%2Fapi%2Ftextapi%2Fahiqar%2F" || $lang || "%2F" || $msUri || "%2F" || $firstpage || "%2Flatest%2Fitem.json"
    
        let $title := $meta//tgmd:title/text()
        let $xhtmlTitle :=
            element xhtml:h3 {
                codepoints-to-string(64 + $pos) || ".",
                string($title),
                element xhtml:a {
                    attribute href { $url },
                    "🔗"
                }
            }
    
        let $msIdentifier := $teiHeader//tei:msIdentifier
        let $xhtmlMsIdentifier :=
            element xhtml:div {
                attribute class { "msIdentifier" },
                element xhtml:h4 { "Manuscript Identifier" },
                element xhtml:div {
                ($msIdentifier/tei:institution/string(),
                $msIdentifier/tei:idno/string(),
                $msIdentifier/tei:settlement/string() => normalize-space()
            ) => string-join("; ")},
                $msIdentifier//tei:idno[starts-with(., "http")] ! element xhtml:a {
                    attribute class { "link-to-source" },
                    attribute href {string(.)},
                    "Digital Source"
                }
            }
        let $xhtmlMsItem :=
            element xhtml:div {
                attribute class { "msItem" },
                $teiHeader//tei:msItem//text()[normalize-space(.) ne ""]
            => string-join("; ")
            }
    
        let $history := $teiHeader//tei:history
        let $xhtmlHistory :=
            element xhtml:div {
                element xhtml:h4 { "History" },
                element xhtml:ul {
                    attribute class {"history"},
                    element xhtml:li {"type: " || $history//tei:objectType},
                    element xhtml:li {"origin: " || $history//tei:country || ", " || $history//tei:placeName},
                    element xhtml:li {"date: " || $history//tei:date}
                }
            }

        let $listBibl := $teiHeader//tei:listBibl/tei:bibl
        let $xhtmlListBibl :=
            element xhtml:div {
                element xhtml:h4 { "Bibliography" },
                element xhtml:ul {
                    attribute class {"listBibl"},
                    for $bibl in $listBibl
                    return
                        element xhtml:li { normalize-space($bibl) }
                }
            }
    
        let $abstract := $teiHeader//tei:abstract
        let $xhtmlAbstract :=
            element xhtml:div {
                attribute class {"abstract"},
                element xhtml:h4 { "Editor`s statements" },
                element xhtml:ul {
                    for $p in $abstract//tei:p
                    return
                        element xhtml:li { normalize-space($p) }
                }
            }
    
        return
            element xhtml:div {
                attribute class { "manuscript" },
                $xhtmlTitle,
                $xhtmlMsIdentifier,
                $xhtmlMsItem,
                $xhtmlHistory,
                $xhtmlListBibl,
                $xhtmlAbstract
            }
    }
};

(:~
 : Provides a list of collation results
:)
declare function local:make-list($lang as xs:string) {
    
    let $sort := function($input) {
                    switch ($input)
                        case "first_narrative_section" return 1
                        case "second_narrative_section" return 2
                        case "third_narrative_section" return 3
                        case "sayings" return 4
                        case "parables" return 5
                        default return 9}
    
    let $resources := xmldb:get-child-resources("/db/data/collation-results/")
    let $stemmas := 
        $resources
        ! substring-before(., "_result")
        ! replace(., "_first_narrative_section|parables|sayings|second_narrative_section|_third_narrative_section", "")
        ! replace(., "_$", "")
        => sort()
        => distinct-values()
    for $i at $pos in $stemmas[starts-with(., $lang)]
    let $iResources := $resources[starts-with(., $i)]
    return
        element xhtml:div {
            element xhtml:h3 { "Stemma " || $pos },
            element xhtml:div {
                "items included: " || replace($i, "ara-karshuni_|syc_", "")
            },
            element xhtml:ol {
                let $units := distinct-values(($iResources ! substring-after(., $i || "_") ! substring-before(., "_result") )) => sort((), $sort)
                for $iii in $units
                return
                    element xhtml:li {
                        ($iii => tokenize("_")) ! (upper-case(substring(., 1, 1)) || substring(., 2, 999)) => string-join(" "), (: ← camel-case function :)
                        element xhtml:ul {
                            for $file in $iResources[contains(., $iii)]
                            order by $file
                            return
                                element xhtml:li {
                                    element xhtml:a {
                                        attribute href { "/rest/collation-results/" || $file },
                                        $file => substring-after(".") => upper-case()
                                    }
                                }
                        }
                    }
            }
        }
};

declare
    %rest:GET
    %rest:HEAD
    %rest:path("/website/collation")
    %output:method("xml")
function website:collation() {
    element xhtml:div {
        element xhtml:h2 {
            text { "Arabic and Karshuni" }
        },
        local:make-list("ara-karshuni"),
        element xhtml:h2 {
            text { "Syriac" }
        },
        local:make-list("syc")
    }
};